SystemOrganization addCategory: #'Bindings-Base'!
SystemOrganization addCategory: #'Bindings-Tests'!
SystemOrganization addCategory: #'Bindings-Widgets'!

Morph subclass: #BindingsTestObject
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a
	
	<bindable>
	^ self valueOfVariableNamed: #a! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a: anObject
	
	<bindable>
	self setVariableNamed: #a toValue: anObject! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b

	^ b! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b: anObject

	b := anObject! !

BindingsTestObject subclass: #BindingsTestObjectWithCollections
	instanceVariableNames: 'd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestObjectWithCollections methodsFor: 'accessing' stamp: 'topa 1/11/2010 17:56'!
d

	<bindableAsCollection>
	^ self valueOfVariableNamed: #d! !

!BindingsTestObjectWithCollections methodsFor: 'accessing' stamp: 'topa 1/11/2010 17:56'!
d: anObject

	<bindableAsCollection>
	self setVariableNamed: #d toValue: anObject.! !

!BindingsTestObjectWithCollections methodsFor: 'as yet unclassified' stamp: 'topa 1/11/2010 15:15'!
initialize

	self d: OrderedCollection new.! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/18/2010 10:27'!
bind: bindable to: aMorph with: aKeyPath
	"Binds the bindable of the current Morph to
	a model described by model key path"
	
	"Check if name of exposed variable is inside this morph"
	(self exposedBindings includes: bindable) ifTrue: [
		"Inject getter/setter for exposed variable (that updates other notifiers) if not existent"

		"Check for collection binding"
		(self isBindable: bindable eligibleForKeyPath: aKeyPath)
			ifTrue: [
				(self ensureManagedCollection: bindable)
					bind: aKeyPath first
					to: self
					with: aKeyPath allButFirst].

		"Remember morph and keypath"
		(self bindings startObservingKey: bindable notifying: aMorph with: aKeyPath).
		"Init notification mechanism at other morph (object notifies morph notifies me)"
		aMorph bindings startObservingKey: aKeyPath notifying: self with: bindable.
		
		"force initial sync"
		self bindings triggerEvent: #hasChanged with: (self -> bindable)].
! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/29/2009 13:41'!
bindings

	(self assureExtension hasProperty: #bindingsManager) 
		ifFalse: [self assureExtension 
			setProperty: #bindingsManager 
			toValue: (self class bindingsManagerClass new target: self)].
		
	^ self assureExtension valueOfProperty: #bindingsManager! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/15/2010 09:26'!
ensureManagedCollection: bindable

	| mc |
	mc := self instVarAt: (self class
		instVarIndexFor: bindable asString
		ifAbsent: [self managedCollectionInExtenstion: bindable]).
			
	(mc isNil or: [mc isManagedCollection not])
		"we rely on the fact that if the bindable is no InstVar, 
		#managedCollectionInExtenstion: will return an instance of ManagedCollection"
		ifTrue: [
			mc := (self class managedCollectionClass wrapping: mc).
			self instVarNamed: bindable put: mc].
	^ mc! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/27/2009 16:07'!
exposedBindings

	^ self class exposedBindings
! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/18/2010 10:15'!
isBindable: bindable eligibleForKeyPath: aKeyPath

	^ (aKeyPath isString not) and:  "collection key paths should not be pure strings/symbols"
		[(aKeyPath isCollection and:  [aKeyPath size > 1]) " col. bindinges only nmake sense for longer key paths"
			and: [((self class exposedCollectionBindings includes: bindable) or:  "the bindee should be a (bindable) collection"
					[(self instVarNamedOrNil: bindable) isCollection])]]! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/12/2010 08:22'!
managedCollectionInExtenstion: bindable

	| propName |
	propName :=  (bindable, ' managedCollection').
	
	(self assureExtension hasProperty: propName) 
		ifFalse: [self assureExtension 
			setProperty: propName 
			toValue: (self class managedCollectionClass wrapping: (self -> bindable))].
		
	^ self assureExtension valueOfProperty: propName! !

!Morph methodsFor: '*bindings' stamp: 'AK 1/4/2010 22:33'!
setVariableNamed: aSymbol toValue: anObject

	
	(anObject = (self instVarNamed: aSymbol))
		ifFalse: [
			self bindings triggerEvent: #aboutToChange with: (self -> aSymbol).
			self instVarNamed: aSymbol put: anObject.
			self bindings triggerEvent: #hasChanged with: (self -> aSymbol)].! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/4/2010 15:51'!
unbind: bindable from: aMorph with: aKeyPath

	(self exposedBindings includes: bindable) ifTrue: [
		"Remember morph and keypath"
		(aMorph exposedBindings includes: aKeyPath) ifTrue: [
				aMorph bindings stopObservingKey: aKeyPath regarding: self with: bindable].
		(self bindings stopObservingKey: bindable regarding: aMorph with: aKeyPath)]! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/11/2010 15:14'!
valueOfVariableNamed: aSymbol

	self bindings triggerEvent: #accessed with: (self -> aSymbol).
	^ self instVarNamed: aSymbol! !

Collection subclass: #ManagedCollection
	instanceVariableNames: 'actualCollection bindingsManager selection lateBound'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

ManagedCollection class
	instanceVariableNames: 'knownMutators'!

ManagedCollection class
	instanceVariableNames: 'knownMutators'!

!ManagedCollection class methodsFor: 'accessing' stamp: 'topa 1/12/2010 08:50'!
knownMutators

	knownMutators ifNil: [
		self knownMutators: 
			(IdentitySet withAll: (
				(Collection organization listAtCategoryNamed: #adding), 
				(Collection organization listAtCategoryNamed: #removing),
				{#atNewIndex:put: . #at:put: . #at:ifAbsentPut: }, "the indexed ones"
				{#associationDeclareAt: . #member . #removeKey: . #removeKey:ifAbsent: . #noCheckNoGrowFillFrom:}, "unordered specials"
				{#add:after: . #add:afterIndex: . #add:before: . #add:beforeIndex: . #addFirst: . #addLast: . 
				 #addAllFirst: . #addAllFirstUnlessAlreadyPresent: . #addAllLast: . #insert:before: . 
				 #removeFirst . #removeFirst: . #removeLast . #removeLast: . #removeAt: .  #removeIndex: .
				 #grow . #setCollection: . #setContents: . #reverseInPlace }, "sequencable friends"
				{#sortTopologically . #reSort  . #sort:to: } "sorters"
			))].
	^ knownMutators! !

!ManagedCollection class methodsFor: 'accessing' stamp: 'topa 1/11/2010 22:06'!
knownMutators: aCollection

	knownMutators := aCollection.! !

!ManagedCollection class methodsFor: 'as yet unclassified' stamp: 'topa 1/11/2010 22:36'!
wrapping: aCollection

	^ self new actualCollection: aCollection! !

!ManagedCollection methodsFor: 'private' stamp: 'topa 1/12/2010 08:29'!
actualCollection

	lateBound ifTrue: [
		^ actualCollection key perform: actualCollection value].
	^ actualCollection! !

!ManagedCollection methodsFor: 'private' stamp: 'topa 1/12/2010 08:27'!
actualCollection: aCollection

	(actualCollection isKindOf: Association) ifTrue: [lateBound := true].
	actualCollection :=  aCollection.! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 1/18/2010 10:00'!
add: anObject

	| ret |
	self bindings triggerEvent: #aboutToChange with: (self -> #arrangedObjects).
	ret := self actualCollection add: anObject.
	self bindings triggerEvent: #hasChanged with: (self -> #arrangedObjects).
	^ ret! !

!ManagedCollection methodsFor: 'accessing' stamp: 'topa 1/11/2010 18:06'!
arrangedObjects

	<bindableAsCollection>
	self bindings triggerEvent: #accessed with: (self -> #arrangedObjects).
	^ self actualCollection! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 1/18/2010 10:00'!
at: aKey put: anObject

	| ret |
	self bindings triggerEvent: #aboutToChange with: (anObject).
	ret := self actualCollection at: aKey put: anObject.
	self bindings triggerEvent: #hasChanged with: (anObject).
	^ ret! !

!ManagedCollection methodsFor: 'bindings' stamp: 'topa 1/12/2010 09:02'!
bind: bindable to: aMorphOrCollection with: aKeyPath

		"Remember morph and keypath"
		(self bindings startObservingKey: bindable notifying: aMorphOrCollection with: aKeyPath).
		"Init notification mechanism at other morph (object notifies morph notifies me)"
		aMorphOrCollection bindings startObservingKey: aKeyPath notifying: self with: bindable! !

!ManagedCollection methodsFor: 'bindings' stamp: 'topa 1/11/2010 18:13'!
bindings

	bindingsManager ifNil: [bindingsManager := self class bindingsManagerClass new target: self].
	^ bindingsManager! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 1/18/2010 10:00'!
do: aBlock

	self bindings triggerEvent: #accessed with: (self -> #arrangedObjects).
	^ self actualCollection do: aBlock.
! !

!ManagedCollection methodsFor: 'error handling' stamp: 'AK 1/17/2010 16:47'!
doesNotUnderstand: aMessage

	(self class knownMutators includes: aMessage selector)
		ifTrue: [
			| result |	
			self bindings triggerEvent: #aboutToChange with: (self actualCollection "????" -> aMessage selector).
			result := self actualCollection perform: aMessage selector withArguments: aMessage arguments.
			self bindings triggerEvent: #hasChanged with: (self actualCollection "????" -> aMessage selector).
			^ result	].
	^ self actualCollection perform: aMessage selector withArguments: aMessage arguments ! !

!ManagedCollection methodsFor: 'initialize-release' stamp: 'topa 1/12/2010 08:28'!
initialize

	lateBound := false.! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 1/18/2010 10:00'!
remove: anObject ifAbsent: aBlock

	| ret |
	self bindings triggerEvent: #aboutToChange with: (self -> #arrangedObjects).
	ret := self actualCollection remove: anObject ifAbsent: aBlock.
	self bindings triggerEvent: #hasChanged with: (self -> #arrangedObjects).
	^ ret! !

!ManagedCollection methodsFor: 'accessing' stamp: 'topa 1/11/2010 18:10'!
selection

	<bindable>
	^ self valueOfVariableNamed: #selection! !

!ManagedCollection methodsFor: 'accessing' stamp: 'topa 1/11/2010 18:09'!
selection: anObject

	<bindable>
	self setVariableNamed: #selection toValue: anObject.! !

PluggableListMorph subclass: #SelectableListMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Widgets'!

!SelectableListMorph methodsFor: 'accessing' stamp: 'topa 1/17/2010 20:21'!
list

	<bindableAsCollection>
	^ self valueOfVariableNamed: #list! !

!SelectableListMorph methodsFor: 'accessing' stamp: 'topa 1/17/2010 20:23'!
list: anObject

	<bindableAsCollection>
	self setVariableNamed: #list toValue: anObject.
	self updateList.
	! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 1/17/2010 20:26'!
selection

	<bindable>
	"bindings stub"
	self bindings triggerEvent: #accessed with: (self -> #selection).
	self selectionIndex = 0 ifTrue: [ ^nil ].
	list ifNotNil: [ ^list at: self selectionIndex ].
	^ self getListItem: self selectionIndex! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 1/17/2010 20:27'!
selection: item
	"Called from outside to request setting a new selection."

	<bindable>
	"bindings stub"
	self bindings triggerEvent: #aboutToChange with: (self -> #selection).
	self selectionIndex: (self getList indexOf: item).
	self bindings triggerEvent: #hasChanged with: (self -> #selection).
! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 1/17/2010 20:39'!
selectionIndex

	<bindable>
	"bindings stub"
	self bindings triggerEvent: #accessed with: (self -> #selectionIndex).
	"return the index we have currently selected, or 0 if none"
	^self listMorph selectedRow ifNil: [ 0 ]! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 1/17/2010 20:43'!
selectionIndex: index
	"Called from outside to request setting a new selection."

	<bindable>
	| row |
	"bindings stub"
	self bindings triggerEvent: #aboutToChange with: (self -> #selection).

	"Called internally to select the index-th item."
	self unhighlightSelection.
	row := index ifNil: [ 0 ].
	row := row min: self getListSize.  "make sure we don't select past the end"
	self listMorph selectedRow: row.
	self highlightSelection.
	self scrollSelectionIntoView.
	
	self bindings triggerEvent: #hasChanged with: (self -> #selection).
! !

Object subclass: #BindingsDescriptor
	instanceVariableNames: 'origin target bindable keyPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsDescriptor class methodsFor: 'instance creation' stamp: 'topa 12/29/2009 16:50'!
origin: aMorph
bindable: aSymbol
target: anotherMorph
keyPath: aKeyPath

	^ self new
		origin: aMorph;
		bindable: aSymbol;
		target: anotherMorph;
		keyPath: aKeyPath! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 12/29/2009 16:55'!
= aBindingsDescriptor

	^ self == aBindingsDescriptor
		ifTrue: [true]
		ifFalse:[
			(self origin     = aBindingsDescriptor origin) and: [
			(self bindable = aBindingsDescriptor bindable) and: [
			(self target    = aBindingsDescriptor target) and: [
			(self keyPath  = aBindingsDescriptor keyPath)]]]]
			! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
bindable

	^ bindable! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
bindable: anObject

	bindable := anObject.! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 1/4/2010 15:45'!
complement

	^ self class
		origin: self target
		bindable: self keyPath
		target: self origin
		keyPath: self bindable
! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 12/29/2009 17:00'!
hash

	^ (((self origin hash 
		bitXor: self bindable hash) 
			bitXor: self target hash)
				bitXor: self keyPath hash)! !

!BindingsDescriptor methodsFor: 'testing' stamp: 'topa 12/29/2009 17:01'!
isBindingDescriptor

	^ true! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath

	^ keyPath! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath: anObject

	keyPath := anObject.! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
origin

	^ origin! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:34'!
origin: anObject

	origin := anObject.! !

!BindingsDescriptor methodsFor: 'printing' stamp: 'topa 12/29/2009 18:43'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self origin; nextPutAll: ''; print: self bindable;
		nextPutAll: ' -> ';
		print: self target; nextPutAll: ''; print: self keyPath;
		nextPut: $)! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
target

	^ target! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
target: anObject

	target := anObject.! !

Object subclass: #BindingsManager
	instanceVariableNames: 'observedBindings target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsManager methodsFor: 'reacting' stamp: 'topa 1/11/2010 15:13'!
aboutToChange: aBindingAssoc! !

!BindingsManager methodsFor: 'reacting' stamp: 'topa 1/11/2010 15:13'!
accessed: aBindingAssoc! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 1/18/2010 10:39'!
allObservedBindings

	^ Array streamContents: [ :stream |
		self observedBindings valuesDo: [:bindings |
			stream nextPutAll: bindings]]! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 1/18/2010 10:43'!
bindingDescriptorForAssociation: aBindingAssoc

	|  target keyPath |
	target := aBindingAssoc key.
	keyPath := aBindingAssoc value.

	^ self allObservedBindings 
		detect: [:bd| 
			(bd target = target) and: [
			(bd keyPath = keyPath) and: [
			bd origin = self target]]]
		ifNone: [nil]! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/29/2009 17:19'!
forgetBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor bindable) 
		remove: aBindingsDescriptor ifAbsent: ["fallthough"].
	self removeIfObsoleteKey: aBindingsDescriptor bindable.
! !

!BindingsManager methodsFor: 'reacting' stamp: 'topa 1/18/2010 10:44'!
hasChanged: aBindingAssoc

	| descriptor messageName|
	descriptor := (self bindingDescriptorForAssociation: aBindingAssoc)
		ifNil: ["nothing to do" ^ self].

	" simple case"
	messageName := descriptor bindable asMutator.
	self target perform: messageName with: (descriptor target perform: descriptor keyPath).
	
"		setVariableNamed: 
		toValue: (descriptor target valueOfVariableNamed: descriptor bindable)."! !

!BindingsManager methodsFor: 'initialize-release' stamp: 'topa 12/27/2009 18:12'!
initialize

	self observedBindings: Dictionary new.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 1/11/2010 15:13'!
installEventHandlers: aBindingsDescriptor

	#( accessed aboutToChange hasChanged ) do: [:eventSymbol |
		aBindingsDescriptor target bindings 
			when: eventSymbol
			send: eventSymbol asMutator
			to: aBindingsDescriptor origin bindings].! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/27/2009 14:17'!
observedBindings

	^ observedBindings! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/27/2009 14:17'!
observedBindings: anObject

	observedBindings := anObject.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:58'!
observedKey: aKey

	^ self observedBindings at: aKey ifAbsentPut: [Set new]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 12/29/2009 17:12'!
recordBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor bindable) 
		add: aBindingsDescriptor.! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 1/11/2010 15:14'!
removeEventHandlers: aBindingsDescriptor

	#( accessed aboutToChange hasChanged ) do: [:eventSymbol |
			aBindingsDescriptor target bindings 
				removeActionsWithReceiver: (aBindingsDescriptor origin bindings) 
				forEvent: eventSymbol].
! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/27/2009 18:10'!
removeIfObsoleteKey: aKey

	((self observedBindings includesKey: aKey) and: [
		(self observedBindings at: aKey) isEmpty])
			ifTrue: [	self observedBindings removeKey: aKey]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 1/18/2010 10:03'!
startObservingKey: bindable notifying: aMorph with: aKeyPath

	| descriptor |
	descriptor := (BindingsDescriptor origin: (self target) bindable: bindable target: aMorph keyPath: aKeyPath).
	self
		recordBinding: descriptor;
		installEventHandlers: descriptor.
! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/29/2009 17:21'!
stopObservingKey: bindable regarding: aMorph with: aKeyPath

	| descriptor |
	descriptor := (BindingsDescriptor origin: (self target) bindable: bindable target: aMorph keyPath: aKeyPath).

	(self observedBindings includesKey: bindable) ifTrue: [
		self 
			forgetBinding: descriptor;
			removeEventHandlers: descriptor]! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 12:45'!
target

	^ target! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 12:45'!
target: aMorph

	target := aMorph! !

!Object class methodsFor: '*bindings' stamp: 'topa 1/11/2010 18:17'!
bindingsManagerClass

	^ BindingsManager! !

!Object class methodsFor: '*bindings' stamp: 'topa 1/11/2010 18:17'!
exposedBindings
	"exposed Bindings are Methods that have the #bindable pragma"
	
	^ Array streamContents: [ :stream |
		(Pragma allNamed: #bindable in: self) do: [:pragma |
			stream nextPut: pragma selector].
		stream nextPutAll: self exposedCollectionBindings]
! !

!Object class methodsFor: '*bindings' stamp: 'topa 1/11/2010 18:18'!
exposedCollectionBindings
	"exposed Collection  Bindings are Methods that have the #bindableAsCollection pragma"

	^ Array streamContents: [ :stream |
		(Pragma allNamed: #bindableAsCollection in: self) do: [:pragma |
			stream nextPut: pragma selector]]! !

!Object class methodsFor: '*bindings' stamp: 'topa 1/12/2010 08:23'!
managedCollectionClass

	^ ManagedCollection! !

!Object methodsFor: '*bindings' stamp: 'topa 1/11/2010 18:29'!
instVarNamedOrNil: name

	^ self instVarAt: (self class
							instVarIndexFor: name asString
							ifAbsent: [nil])! !

!Object methodsFor: '*bindings' stamp: 'topa 12/29/2009 17:01'!
isBindingDescriptor

	^ false! !

!Object methodsFor: '*bindings' stamp: 'topa 1/12/2010 08:59'!
isManagedCollection

	^ false! !

TestCase subclass: #BindingsTestCase
	instanceVariableNames: 'sut helper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

BindingsTestCase subclass: #BindingsCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsCollectionTest methodsFor: 'running' stamp: 'topa 1/11/2010 15:18'!
setUp

	self 
		sut: BindingsTestObjectWithCollections new;
		helper: BindingsTestObjectWithCollections new.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'AK 1/17/2010 17:20'!
testArrayKeyPath

	| bindingsSize |
	bindingsSize := self sut bindings observedBindings size.
	
	self sut bind: #d to: self helper with: #(d a).
	self should: [bindingsSize + 2 =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to establish'.
	
	self sut unbind: #d from: self helper with: #(d a).
	self should: [bindingsSize =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to remove'.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'topa 1/11/2010 17:14'!
testCreationRemovalBind

	| sutDSize helperDSize |
	self sut bind: #d to: self helper with: #d.
	
	sutDSize := self sut d size.
	helperDSize := self helper d size.
	
	self sut d add: 1.
	self should: [(sutDSize + 1 = self sut d size) and: [helperDSize + 1 = self helper d size]]	
		description: 'Bound colections should be both one larger after addition'.

	self helper d remove: 1.
	self should: [(self sut d size = sutDSize) and: [ self helper d size = helperDSize]]	
		description: 'Bound colections should be both one smaller after removal'.

	self sut unbind: #d from: self helper with: #d.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'topa 1/15/2010 11:22'!
testKeyPathSemantics

	| bindingsSize |
	bindingsSize := self sut bindings observedBindings size.
	
	self sut bind: #d to: self helper with: #(d a).
	self should: [bindingsSize + 1 =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to establish'.
	
	self sut unbind: #d from: self helper with: #(d a).
	self should: [bindingsSize =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to remove'.! !

BindingsTestCase subclass: #BindingsDescriptorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsDescriptorTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self
		sut: BindingsDescriptor new;
		helper: BindingsTestObject new.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testDescriptorCreation

	| bindings |
	self helper bind: #a to: self helper with: #a.
	
	bindings := self helper bindings observedBindings at: #a.
	bindings do: [:binding | 
		self should: [binding isBindingDescriptor]
				description: 'Binding descriptors should be created automatically upon binding creation'].
	
	self helper unbind: #a from: self helper with: #a.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:22'!
testDescriptorEquality

	| comparand |
	self sut 
		origin: self helper;
		bindable: #a;
		target: self helper;
		keyPath: #a.
		
	comparand := BindingsDescriptor origin: self helper bindable: #a target: self helper keyPath: #a.
	
	self should: [(self sut = comparand) and: [self sut hash = comparand hash]]
			description: 'Content-equal Bindings-Descriptors should be treated as equal';
		shouldnt: [self sut == comparand]
			description: 'Content-equal Bindings-Descriptors should not be treated as identical'.
	! !

BindingsTestCase subclass: #BindingsManagedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsManagedCollectionTest methodsFor: 'running' stamp: 'topa 1/11/2010 17:17'!
setUp

	super setUp.
	self 
		sut: ManagedCollection new;
		helper: ManagedCollection new.
	! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testAddToCollection
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	
	self 
		should: [self helper first a = 10];
		should: [self helper second b = 20].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:20'!
testHasArrangedObjects

	self
		should: [self sut respondsTo: #arrangedObjects] description: 'A Managed Collection should provide all arrangedObjects';
		should: [self sut class exposedBindings includes: #arrangedObjects] description: 'The arranged Objects should be bindable'.
! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:04'!
testHasBindingsManager

	self
		should: [self sut respondsTo: #bindings] 
			description: 'A Managed Collection should have a Bindigns manager'.
		! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:20'!
testHasSelection

	self
		should: [self sut respondsTo: #selection] description: 'A Managed Collection should provide a selection';
		should: [self sut class exposedBindings includes: #selection] description: 'A selection should be bindable'.
		! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testManagedCollection
	| sutFirst sutSecond |

	self sut: BindingsTestCollection new.
	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	self sut selectObject: sutFirst.
	
	self should: [self helper selection a = 10].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testReadFromCollectionChange
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self helper add: sutFirst.
	self helper add: sutSecond.
	
	self 
		should: [self sut first a = 10];
		should: [self sut second a = 20].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testRemoveFromCollection
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	self sut remove: sutFirst.
	
	self 
		should: [self helper first a = 20];
		should: [self helper size = 1].! !

BindingsTestCase subclass: #BindingsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self 
		sut: BindingsTestObject new;
		helper: BindingsTestObject new.! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testChainedBinding
	
	| theValue secondHelper |
	theValue := 42.
	secondHelper := BindingsTestObject new.
	self sut bind: #a to: self helper with: #a.
	self helper bind: #a to: secondHelper with: #a.
	
	secondHelper a: theValue.
	self should: [(self helper a = theValue) and: [self sut a = theValue]]
		description: 'Bound Morphs should have a value set by the binding target, even chained'.
	
	self sut unbind: #a from: self helper with: #a.
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testCreateRemoveBindings
	
	| numOfBindings numOfBindingsTwoWay |
	numOfBindings := self sut bindings observedBindings size.
	numOfBindingsTwoWay := self helper bindings observedBindings size.
	
	
	self sut bind: #a to: self helper with: #a.	
	self should: [self sut bindings observedBindings size = (numOfBindings + 1)]
		description: 'Binding Morphs should increase the size of the observed bindings'.
	self should: [self sut bindings target = self sut]
		description: 'Binding Manager should know its target.'.
	self should: [self helper bindings observedBindings size = (numOfBindingsTwoWay + 1)]
		description: 'Two Way Binding Morphs should increase the size of the observed bindings at the target, too'.
		

	self sut unbind: #a from: self helper with: #a.
	self should: [self sut bindings observedBindings size = numOfBindings]
		description: ' Unbinding Morphs should decrease the size of the observed bindings'.	
	self should: [self helper bindings observedBindings size = numOfBindingsTwoWay]
		description: ' Unbinding Morphs should decrease the size of the observed bindings at the target in a two way binding'.	
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:21'!
testExposedBindings

	self 
		should: [self sut class exposedBindings includes: #a];
		should: [self sut class exposedBindings includes: #a:];
		should: [self sut class exposedBindings size = 2];
		shouldnt: [self sut class exposedBindings includes: #b];
		shouldnt: [self sut class exposedBindings includes: #b:].
	! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testReadBinding
	
	| theValue |
	theValue := 42.
	self sut bind: #a to: self helper with: #a.
	self helper a: theValue.	
	self should: [self sut a = theValue]
		description: 'Bound Morphs should have a value set by the binding target'.
	
	self sut unbind: #a from: self helper with: #a.
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testReadWriteBinding
	
	| theValue |
	theValue := 42.
	self sut bind: #a to: self helper with: #a.
	self helper a: theValue.	
	self should: [self sut a = theValue]
		description: 'Bound Morphs should have a value set by the binding target'.
	self should: [self helper a = theValue]
		description: 'Bound Morphs should should not change their value unconditionally'.
	theValue := 16.
	self sut a: theValue.
	self should: [self helper a = theValue]
		description: 'Bound Targets should have a value set by the binding Morphs, too'.
	
	self sut unbind: #a from: self helper with: #a.! !

!BindingsTestCase class methodsFor: 'testing' stamp: 'topa 12/29/2009 18:24'!
isAbstract

	^self name = #BindingsTestCase! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper

	^ helper! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper: aTestHelperObject

	helper := aTestHelperObject.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:16'!
setUp

	super setUp.
	self
		sut: nil;
		helper: nil.! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
sut

	^ sut! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:14'!
sut: aSystemUnderTest

	sut := aSystemUnderTest.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:13'!
tearDown

	self
		sut: nil;
		helper: nil.
	super tearDown! !

OrderedCollection subclass: #BindingsTestCollection
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestCollection methodsFor: 'binding methods' stamp: 'AK 1/6/2010 10:49'!
selectObject: anObject
      "The programmer has to expose this on binder side"
	self bindings triggerEvent: #objectSelected with: anObject.! !
